do
#
# Terran Fragmentation/Sensor Shells
#
local EffectTemplate = import('/lua/EffectTemplates.lua')
local TArtilleryProjectile = import('/lua/terranprojectiles.lua').TArtilleryProjectile
local RandomFloat = import('/lua/utilities.lua').GetRandomFloat
local VizMarker = import('/lua/sim/VizMarker.lua').VizMarker

TIFFragmentationSensorShellForMavor = Class(TArtilleryProjectile) {
    OnImpact = function(self, TargetType, TargetEntity) 
        local FxFragEffect = EffectTemplate.TFragmentationSensorShellFrag 
        local ChildProjectileBP = '/projectiles/TIFHETacticalNuclearShell01/TIFHETacticalNuclearShell01_proj.bp'
        local SecondaryChildProjectileBP = '/projectiles/ADFQuantumCannon01/ADFQuantumCannon01_proj.bp'
        --local SecondaryChildProjectileBP =  '/projectiles/TIFMissileCruise01/TIFMissileCruise01_proj.bp'

       
        # Split effects
        for k, v in FxFragEffect do
            CreateEmitterAtEntity( self, self:GetArmy(), v )
        end
        
        local vx, vy, vz = self:GetVelocity()
        local velocity = 6
    
        # One initial projectile following same directional path as the original (Removed)
        self:CreateChildProjectile(ChildProjectileBP):SetVelocity(vx, vy, vz):SetVelocity(velocity):PassDamageData(self.DamageData)

#--------------Primary Explosions-------------
        # Create several other projectiles in a dispersal pattern
        local numProjectiles = 3
        local angle = (2*math.pi) / numProjectiles
        local angleInitial = RandomFloat( 0, angle )
        
        # Randomization of the spread
        local angleVariation = angle * 0.35 # Adjusts angle variance spread
        local spreadMul = 5 # Adjusts the width of the dispersal        
        
        local xVec = 0 
        local yVec = vy
        local zVec = 0
        local projVelocity = 0

        # Launch projectiles at semi-random angles away from split location
        for i = 0, (numProjectiles -1) do
            projVelocity = RandomFloat(5,15)
            xVec = vx*2 + (math.sin(angleInitial + (i*angle) + RandomFloat(-angleVariation, angleVariation))) * spreadMul
            zVec = vz*2 + (math.cos(angleInitial + (i*angle) + RandomFloat(-angleVariation, angleVariation))) * spreadMul
            local proj = self:CreateChildProjectile(ChildProjectileBP)
            proj:SetVelocity(xVec,yVec,zVec)
            proj:SetVelocity(projVelocity)
            proj:PassDamageData(self.DamageData)                        
        end

#--------------Secondary Explosions---------
        # Create several other projectiles in a dispersal pattern
        numProjectiles = 10
        angle = (2*math.pi) / numProjectiles
        angleInitial = RandomFloat( 0, angle )
        
        # Randomization of the spread
        angleVariation = angle * 0.35 # Adjusts angle variance spread
        spreadMul = 1000 # Adjusts the width of the dispersal        
        
        xVec = 0 
        yVec = vy
        zVec = 0
        projVelocity = 0

#Trying to set up secondary smaller damage explosions - does 1/4 damage
        local SecondaryDamageData = {
        DamageRadius = 1 ,
        --DamageRadius = self.DamageData.DamageRadius / 7 ,
        DamageAmount = self.DamageData.DamageAmount / 4,
        DamageType = self.DamageData.DamageType,
        DamageFriendly = self.DamageData.DamageFriendly ,
        }

        # Launch projectiles at semi-random angles away from split location
        for i = 0, (numProjectiles -1) do
            projVelocity = RandomFloat(1,4)
            xVec = vx*2 + (math.sin(angleInitial + (i*angle) + RandomFloat(-angleVariation, angleVariation))) * spreadMul
            zVec = vz*2 + (math.cos(angleInitial + (i*angle) + RandomFloat(-angleVariation, angleVariation))) * spreadMul
            local proj = self:CreateChildProjectile(SecondaryChildProjectileBP )
            proj:SetVelocity(xVec,yVec,zVec)
            proj:SetVelocity(projVelocity)
            proj:PassDamageData(SecondaryDamageData)                        
        end


        #Reveal the map where the shell splits.
        local pos = self:GetPosition()
        local spec = {
            X = pos[1],
            Z = pos[3],
            Radius = self.Data.Radius,
            LifeTime = self.Data.Lifetime,
            Army = self.Data.Army,
            Omni = false,
            WaterVision = false,
        }
        local vizEntity = VizMarker(spec)
       
        self:Destroy()
    end,
}

TypeClass = TIFFragmentationSensorShellForMavor
end