local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Group = import('/lua/maui/group.lua').Group
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local Button = import('/lua/maui/button.lua').Button
local ToolTip = import('/lua/ui/game/tooltip.lua')
local Grid = import('/lua/maui/grid.lua').Grid
local Dragger = import('/lua/maui/dragger.lua').Dragger

MainButton = false

function CreateButton(unit)
    if MainButton then
        MainButton:Destroy()
    end
    local oldUpgButton = import('/lua/ui/game/construction.lua').controls.enhanceButton
    MainButton = Button(oldUpgButton, '/mods/SupremilatedATWMediaSC/textures/combat_up.dds', '/mods/SupremilatedATWMediaSC/textures/combat_down.dds', '/mods/SupremilatedATWMediaSC/textures/combat_over.dds', '/mods/SupremilatedATWMediaSC/textures/combat_up.dds', "UI_Menu_MouseDown_Sml", "UI_Menu_MouseDown_Sml")
    LayoutHelpers.FillParent(MainButton, oldUpgButton)
    MainButton.OnClick = function(self, modifiers)
        ConfigureUpgrades(unit)
    end
end

--configuration window
function ConfigureUpgrades(unit)
    --unit is the quantum gate being set, backupEnhanceList is for cancelling
    if not unit.EnhanceList then
        unit.EnhanceList = {}
    end
    local backupEnhanceList = table.copy(unit.EnhanceList)

    local window = Bitmap(GetFrame(0))
    window:SetTexture('/mods/SupremilatedATWMediaSC/textures/configwindow.dds')
    LayoutHelpers.AtCenterIn(window, GetFrame(0))
    window.Depth:Set(1000)
    local buttonGrid = Grid(window, 48, 48)
    LayoutHelpers.AtLeftTopIn(buttonGrid, window, 10, 30)
    buttonGrid.Right:Set(function() return window.Right() - 10 end)
    buttonGrid.Bottom:Set(function() return window.Bottom() - 32 end)
    buttonGrid.Depth:Set(window.Depth() + 10)

    --the icons showing which upgrades the scu will recieve
    buttonGrid.QueuedUpgrades = {}
    for i = 1, 10 do
        local index = i
        buttonGrid.QueuedUpgrades[index] = Bitmap(buttonGrid)
        buttonGrid.QueuedUpgrades[index]:SetTexture('/mods/SupremilatedATWMediaSC/textures/queueborder.dds')
        if index == 1 then
            LayoutHelpers.AtLeftTopIn(buttonGrid.QueuedUpgrades[index], window, 4, 4)
        else
            LayoutHelpers.RightOf(buttonGrid.QueuedUpgrades[index], buttonGrid.QueuedUpgrades[index-1])
        end
    end

    local okButton = UIUtil.CreateButtonStd(window, '/widgets/small', 'OK', 16)
    LayoutHelpers.AtLeftTopIn(okButton, window, 160, 123)
    okButton.OnClick = function(self)
        SimCallback({Func = 'SetGateEnhanceList', Args = {List = unit.EnhanceList, ID = unit:GetEntityId()}})
        window:Destroy()
    end
    local cancelButton = UIUtil.CreateButtonStd(window, '/widgets/small', 'Cancel', 16)
    LayoutHelpers.AtLeftTopIn(cancelButton, window, 8, 123)
    cancelButton.OnClick = function(self)
        unit.EnhanceList = table.copy(backupEnhanceList)
        window:Destroy()
    end

    LayoutGrid(buttonGrid, unit)
end

--shows available and current enhancements for an scu type
function LayoutGrid(buttonGrid, unit)
    --get the enhancements available to whichever scu is being edited
    local bpid = 'ual0301'
    local factoryBP = unit:GetBlueprint()
    for i, cat in factoryBP.Categories do
        if cat == 'CYBRAN' then
            bpid = 'url0301'
        elseif cat == 'UEF' then
            bpid = 'uel0301'
        end
    end
    local bp = __blueprints[bpid]
    local availableEnhancements = bp["Enhancements"]

    --clear the current enhancements, and add the new ones
    for i, v in buttonGrid.QueuedUpgrades do
        if v.Icon then
            v.Icon:Destroy()
            v.Icon = false
        end
        if unit.EnhanceList[i] then
            local enhancementName = unit.EnhanceList[i]
            v.Icon = CreateEnhancementButton(v, enhancementName, availableEnhancements[enhancementName], bpid, 22, unit, buttonGrid)
            v.Icon.enhancementName = enhancementName
            v.Icon.Index = i
            LayoutHelpers.AtCenterIn(v.Icon, v)
        end
    end

    --make a table of available enhancements, not showing any that are already owned, any that need a non queued prerequisite, or any where the slot is already used
    buttonGrid:DeleteAndDestroyAll(true)
    local visCols, visRows = buttonGrid:GetVisible()
    local currentRow = 1
    local currentCol = 1
    buttonGrid:AppendCols(visCols, true)
    buttonGrid:AppendRows(1, true)
    local index = 0
    local tempAvailableButtons = {}
    for name, data in availableEnhancements do
        local alreadyOwns = false
        for i, v in buttonGrid.QueuedUpgrades do
            if v.Icon.enhancementName then
                if v.Icon.enhancementName == name then
                    alreadyOwns = true
                end
            end
        end
        if not alreadyOwns then
            if data['Slot'] and not string.find(name, 'Remove') then
                if data['Prerequisite'] then
                    for i, v in buttonGrid.QueuedUpgrades do
                        if v.Icon.enhancementName then
                            if v.Icon.enhancementName == data['Prerequisite'] then
                                table.insert(tempAvailableButtons, {Name = name, Enhancement = data})
                            end
                        end
                    end
                else
                    local slotUsed = false
                    for i, v in buttonGrid.QueuedUpgrades do
                        if v.Icon.enhancementName then
                            if availableEnhancements[v.Icon.enhancementName].Slot == data['Slot'] then
                                slotUsed = true
                            end
                        end
                    end
                    if not slotUsed then
                        table.insert(tempAvailableButtons, {Name = name, Enhancement = data})
                    end
                end
            end
        end
    end
    table.sort(tempAvailableButtons, function(up1, up2) return (up1.Enhancement.Slot .. up1.Name) <= (up2.Enhancement.Slot .. up2.Name) end)
    for i, data in tempAvailableButtons do
        local button = CreateEnhancementButton(buttonGrid, data.Name, data.Enhancement, bpid, 46, unit, buttonGrid)
        if button then
            buttonGrid:SetItem(button, currentCol, currentRow, true)
            if currentCol < visCols then
                currentCol = currentCol + 1
            else
                currentCol = 1
                currentRow = currentRow + 1
                buttonGrid:AppendRows(1, true)
            end
        end
    end
    buttonGrid:EndBatch()
end

function GetEnhancementButtonBmpName(bpid, appendString)
    if string.sub(bpid, 2, 2) == 'a' then
        return '/dialogs/enhance-icons-aeon-' .. bpid .. '/'.. appendString
    elseif string.sub(bpid, 2, 2) == 'e' then
        return '/dialogs/enhance-icons-uef-' .. bpid .. '/' .. appendString
    elseif string.sub(bpid, 2, 2) == 'r' then
        return '/dialogs/enhance-icons-cybran-' .. bpid .. '/' .. appendString
    end
end

function CreateEnhancementButton(parent, enhancementName, enhancement, bpid, size, unit, buttonGrid)
    local tempBmpName = ""

    if enhancement.Slot == "Back" then
        tempBmpName = bpid .. "-b-" .. enhancement.Icon
    elseif enhancement.Slot == "LCH" then
        tempBmpName = bpid .. "-la-" .. enhancement.Icon
    elseif enhancement.Slot == "RCH" then
        tempBmpName = bpid .. "-ra-" .. enhancement.Icon
    elseif enhancement.Slot == "Shield" then
        tempBmpName = bpid .. "-" .. enhancement.Icon
    else
        return nil
    end
    
    tempBmpName = GetEnhancementButtonBmpName(bpid, tempBmpName)

    local button = false
    if( string.find( enhancementName, 'Remove' ) ) then
        button = Button(parent,
            UIUtil.UIFile(tempBmpName .. '_btn_sel.dds'),
            UIUtil.UIFile(tempBmpName .. '_btn_over.dds'),
            UIUtil.UIFile(tempBmpName .. '_btn_down.dds'),
            UIUtil.UIFile(tempBmpName .. '_btn_up.dds'),
            "UI_Enhancements_Click", "UI_Enhancements_Rollover")
    else
        button = Button(parent,
            UIUtil.UIFile(tempBmpName .. '_btn_up.dds'),
            UIUtil.UIFile(tempBmpName .. '_btn_over.dds'),
            UIUtil.UIFile(tempBmpName .. '_btn_down.dds'),
            UIUtil.UIFile(tempBmpName .. '_btn_sel.dds'),
            "UI_Enhancements_Click", "UI_Enhancements_Rollover")
    end
    button.Width:Set(size)
    button.Height:Set(size)

    button.OnClick = function(self, modifiers)
        if size == 46 then
            --if it's a main button, find the last free queue space and add the enhancement to the queue
            local nextfree = false
            for i, space in parent.QueuedUpgrades do
                if not space.Icon then
                    nextfree = i
                    break
                end
            end
            if nextfree then
                parent.QueuedUpgrades[nextfree].Icon = CreateEnhancementButton(parent.QueuedUpgrades[nextfree], enhancementName, enhancement, bpid, 22)
                LayoutHelpers.AtCenterIn(parent.QueuedUpgrades[nextfree].Icon, parent.QueuedUpgrades[nextfree])
                parent.QueuedUpgrades[nextfree].Icon.enhancementName = enhancementName
                parent.QueuedUpgrades[nextfree].Icon.Index = nextfree
                unit.EnhanceList[nextfree] = enhancementName
            end
        else
            --if not a main button then remove it from the queue and shift all the higher ones down a spot
            unit.EnhanceList[parent.Icon.Index] = nil
            local firstnil = false
            parent.Icon:Destroy()
            parent.Icon = false
            for i, v in buttonGrid.QueuedUpgrades do
                if firstnil then
                    if v.Icon then
                        buttonGrid.QueuedUpgrades[i-1].Icon = buttonGrid.QueuedUpgrades[i].Icon
                        buttonGrid.QueuedUpgrades[i-1].Icon.Index = i-1
                        unit.EnhanceList[i] = nil
                        unit.EnhanceList[i-1] = buttonGrid.QueuedUpgrades[i].Icon.enhancementName
                        buttonGrid.QueuedUpgrades[i].Icon:Destroy()
                        buttonGrid.QueuedUpgrades[i].Icon = false
                    end
                end
                if not v.Icon then
                    firstnil = true
                end
            end
        end
        LayoutGrid(buttonGrid, unit)
    end

    --if there's a selection then show info for the enhancement
    local testUnit = GetSelectedUnits()
    button.HandleEvent = function(self, event)
        if testUnit then
            if event.Type == 'MouseEnter' then
                import('/lua/ui/game/unitviewDetail.lua').ShowEnhancement(enhancement, bpid, enhancement.Icon, testUnit[1])
            end
        end
        if event.Type == 'MouseExit' then
            import('/lua/ui/game/unitviewDetail.lua').Hide()
        end
        Button.HandleEvent(self, event)
    end
    button:UseAlphaHitTest(false)

    return button
end

--all the other functions are user side, but we need the list sim side too so we can use it. This is only called when the OK button is pressed.
function SetSimSideList(Args)
    local unit = GetEntityById(Args.ID)
    unit.EnhanceList = Args.List
end
